﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using System.Xml.Xsl;

using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Wordprocessing;

using UsingOpenXmlSdkWebForm.Models;

namespace UsingOpenXmlSdkWebForm
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            Website website = GetWebsite();

            WebsiteNameLabel.Text = website.Name;
            WebsiteUrlHyperLink.NavigateUrl = WebsiteUrlHyperLink.Text = website.Url;
            WebsiteStartDate.Text = website.Statistics.StartDate.ToShortDateString();
            WebsiteEndDate.Text = website.Statistics.EndDate.ToShortDateString();

            CountryStatisticsRepeater.DataSource = website.Statistics.CountryStatisticsList;
            CountryStatisticsRepeater.DataBind();

            VisitorStatisticsRepeater.DataSource = website.Statistics.VisitorStatisticsList;
            VisitorStatisticsRepeater.DataBind();
        }

        protected void ExportToWordButton_OnClick(object sender, EventArgs e)
        {
            string xml = GetWebsite().ToXml();
            string templateDocumentPath = string.Format("{0}\\Website.docx", Server.MapPath("~/App_Data"));
            string xsltBodyPath = string.Format("{0}/document.xslt", Server.MapPath("~/App_Data"));
            Stream templateDocumentReadStream = System.IO.File.OpenRead(templateDocumentPath);
            BinaryReader templateDocumentBinaryReader = new BinaryReader(templateDocumentReadStream);
            byte[] templateDocumentByteArray = templateDocumentBinaryReader.ReadBytes(Convert.ToInt32(templateDocumentReadStream.Length));
            MemoryStream templateDocumentStream = new MemoryStream();
            templateDocumentStream.Write(templateDocumentByteArray, 0, templateDocumentByteArray.Length);
            string xsltBody = System.IO.File.ReadAllText(xsltBodyPath);
            XmlDocument xmlBody = new XmlDocument();
            xmlBody.LoadXml(xml);
            GenerateWordDocument(xmlBody.OuterXml, xsltBody, ref templateDocumentStream);

            byte[] fileContent = templateDocumentStream.ToArray();
            templateDocumentStream.Close();

            // Response.Buffer = true;
            Response.ContentType = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
            Response.AddHeader("Content-Disposition", "filename=Website.docx");
            Response.BinaryWrite(fileContent);
            Response.End();
            // return File(fileContent, "application/vnd.openxmlformats-officedocument.wordprocessingml.document", "Website.docx");
        }

        private static void GenerateWordDocument(string xmlBody, string xsltBody, ref MemoryStream templateDocumentStream)
        {
            XmlDocument wordBody = new XmlDocument();

            // Create a writer for the output of the Xsl Transformation.
            StringWriter sw = new StringWriter();
            XmlWriter xw = XmlWriter.Create(sw);

            // Create the Xsl Transformation object.
            XslCompiledTransform transform = new XslCompiledTransform();
            transform.Load(new XmlTextReader(new StringReader(xsltBody)));

            // Transform the xml data into Open XML 2.0 Wordprocessing format.
            transform.Transform(XmlReader.Create(new StringReader(xmlBody)), xw);

            // Create an Xml Document with the new content.
            wordBody.LoadXml(sw.ToString());

            // Use the Open XML SDK version 2.0 to open the output document in edit mode.
            using (WordprocessingDocument output = WordprocessingDocument.Open(templateDocumentStream, true))
            {
                // Using the body element within the new content XmlDocument create
                // a new Open Xml Body object.
                Body updatedBodyContent = new Body(wordBody.DocumentElement.InnerXml);

                // Replace the existing Document Body with the new content.
                output.MainDocumentPart.Document.Body = updatedBodyContent;

                // Close the output document.
                output.Close();
            }
        }

        private Website GetWebsite()
        {
            return new Website
            {
                Name = "ASP.NET Wiki",
                Url = "http://www.aspnetwiki.com",

                Statistics = new Statistics
                {
                    StartDate = new DateTime(2011, 6, 1),
                    EndDate = new DateTime(2011, 6, 9),

                    CountryStatisticsList = new CountryStatistics[]
                    {
                        new CountryStatistics { Name = "United States", Visits = 938, PagesPerVisit = 2.32F, AvgTimeOnSite = new DateTime(2011,6,9,0,2,32) },
                        new CountryStatistics { Name = "India", Visits = 321, PagesPerVisit = 1.26F, AvgTimeOnSite = new DateTime(2011,6,9,0,1,2) },
                        new CountryStatistics { Name = "United Kingdom", Visits = 161, PagesPerVisit = 1.30F, AvgTimeOnSite = new DateTime(2011,6,9,0,0,55) },
                        new CountryStatistics { Name = "Canada", Visits = 113, PagesPerVisit = 1.44F, AvgTimeOnSite = new DateTime(2011,6,9,0,1,28) },
                        new CountryStatistics { Name = "Australia", Visits = 110, PagesPerVisit = 1.38F, AvgTimeOnSite = new DateTime(2011,6,9,0,1,7) },
                        new CountryStatistics { Name = "Brazil", Visits = 105, PagesPerVisit = 1.21F, AvgTimeOnSite = new DateTime(2011,6,9,0,0,40) },
                        new CountryStatistics { Name = "Germany", Visits = 88, PagesPerVisit = 1.48F, AvgTimeOnSite = new DateTime(2011,6,9,0,0,53) },
                        new CountryStatistics { Name = "France", Visits = 66, PagesPerVisit = 1.59F, AvgTimeOnSite = new DateTime(2011,6,9,0,1,0) },
                        new CountryStatistics { Name = "Netherlands", Visits = 64, PagesPerVisit = 1.14F, AvgTimeOnSite = new DateTime(2011,6,9,0,0,33) },
                        new CountryStatistics { Name = "Russia", Visits = 58, PagesPerVisit = 1.95F, AvgTimeOnSite = new DateTime(2011,6,9,0,1,27) }
                    },
                    VisitorStatisticsList = new VisitorStatistics[]
                    {
                        new VisitorStatistics { Date = new DateTime(2011,6,1), AbsoluteUniqueVisitors = 134, PageViews = 212 },
                        new VisitorStatistics { Date = new DateTime(2011,6,2), AbsoluteUniqueVisitors = 123, PageViews = 192 },
                        new VisitorStatistics { Date = new DateTime(2011,6,3), AbsoluteUniqueVisitors = 112, PageViews = 178 },
                        new VisitorStatistics { Date = new DateTime(2011,6,4), AbsoluteUniqueVisitors = 49, PageViews = 112 },
                        new VisitorStatistics { Date = new DateTime(2011,6,5), AbsoluteUniqueVisitors = 44, PageViews = 95 },
                        new VisitorStatistics { Date = new DateTime(2011,6,6), AbsoluteUniqueVisitors = 159, PageViews = 243 },
                        new VisitorStatistics { Date = new DateTime(2011,6,7), AbsoluteUniqueVisitors = 187, PageViews = 276 },
                        new VisitorStatistics { Date = new DateTime(2011,6,8), AbsoluteUniqueVisitors = 148, PageViews = 244 },
                        new VisitorStatistics { Date = new DateTime(2011,6,9), AbsoluteUniqueVisitors = 126, PageViews = 197 }
                    }
                }
            };
        }
    }
}
